#ifndef EP_COMM_HPP_INCLUDED
#define EP_COMM_HPP_INCLUDED

#include "ep_message.hpp"
#include "ep_intercomm.hpp"
#include "ep_barrier.hpp"
#include "ep_buffer.hpp"

typedef std::pair< int, int > SIZE_RANK_INFO; // < rank, size>
typedef std::vector< std::pair<int, int> > RANK_MAP;  // at(ep_rank) = <ep_rank_local, mpi_rank>
typedef std::map<int, std::pair<int, int> > EP_RANK_MAP;  // key(ep_rank) = <ep_rank_local, mpi_rank>
//typedef std::vector<std::pair< std::pair<int, int>, std::pair<int, int> > > INTERCOMM_RANK_MAP;
typedef std::list<ep_lib::MPI_Message > Message_list;

namespace ep_lib
{

  class ep_comm;
  
  class ep_communicator
  {
    public:

    SIZE_RANK_INFO size_rank_info[3]; // 0: ep_rank,     ep_size
                                      // 1: ep_rank_loc, num_ep
                                      // 2: mpi_rank,    mpi_size


    ep_comm **comm_list;

    Message_list *message_queue;


    int comm_label;

    ep_intercomm *intercomm;

    ep_communicator();
    bool operator == (ep_communicator right);
    bool operator != (ep_communicator right);
    
  };
  

  class ep_comm
  {
    public:

    bool is_ep;
    bool is_intercomm;

    BUFFER     *my_buffer;
    ep_barrier *ep_barrier;
    //RANK_MAP   *rank_map;
    EP_RANK_MAP   *ep_rank_map;
    void* mpi_comm;
    ep_communicator *ep_comm_ptr;
    ep_comm    **mem_bridge;
    void* mpi_bridge;

    ep_comm();  
    ep_comm(void* comm);


    bool operator == (ep_comm right);
    bool operator != (ep_comm right);
    bool is_null();

  };
  
  typedef ep_comm*  MPI_Comm;


}



#endif // EP_COMM_HPP_INCLUDED

